package java.net;

import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.util.Iterator;
import java.util.Set;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Method;
import java.lang.reflect.Field;

class HttpConnectSocketImpl extends PlainSocketImpl
{
    private static final String httpURLClazzStr = "sun.net.www.protocol.http.HttpURLConnection";
    private static final String netClientClazzStr = "sun.net.NetworkClient";
    private static final String doTunnelingStr = "doTunneling";
    private static final Field httpField;
    private static final Field serverSocketField;
    private static final Method doTunneling;
    private final String server;
    private InetSocketAddress external_address;
    private HashMap<Integer, Object> optionsMap;
    
    HttpConnectSocketImpl(Proxy proxy) {
        super(false);
        this.optionsMap = new HashMap();
        SocketAddress address = proxy.address();
        if (!(address instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        InetSocketAddress inetSocketAddress = (InetSocketAddress)address;
        this.server = inetSocketAddress.getHostString();
        this.port = inetSocketAddress.getPort();
        return;
    }
    
    @Override
    protected void connect(SocketAddress socketAddress, int n) throws IOException {
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        InetSocketAddress external_address = (InetSocketAddress)socketAddress;
        String s = external_address.isUnresolved() ? external_address.getHostName() : external_address.getAddress().getHostAddress();
        int port = external_address.getPort();
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkConnect(s, port);
        }
        Socket privilegedDoTunnel = this.privilegedDoTunnel("http://" + s + ":" + port, n);
        this.external_address = external_address;
        this.close();
        AbstractPlainSocketImpl impl = (AbstractPlainSocketImpl)privilegedDoTunnel.impl;
        this.getSocket().impl = impl;
        Set<Map.Entry<Integer, Object>> entrySet = this.optionsMap.entrySet();
        try {
            for (Map.Entry entry : entrySet) {
                impl.setOption((int)entry.getKey(), entry.getValue());
            }
        }
        catch (IOException ex) {}
        return;
    }
    
    @Override
    public void setOption(int i, Object value) throws SocketException {
        super.setOption(i, value);
        if (this.external_address != null) {
            return;
        }
        this.optionsMap.put(Integer.valueOf(i), value);
        return;
    }
    
    private Socket privilegedDoTunnel(final String s, final int n) throws IOException {
        try {
            return (Socket)AccessController.doPrivileged(new PrivilegedExceptionAction<Socket>() {
                @Override
                public Socket run() throws IOException {
                    return HttpConnectSocketImpl.access$200(HttpConnectSocketImpl.this, s, n);
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw (IOException)ex.getException();
        }
    }
    
    private Socket doTunnel(String s, int connectTimeout) throws IOException {
        HttpURLConnection obj = (HttpURLConnection)new URL(s).openConnection(new Proxy(Proxy.Type.HTTP, new InetSocketAddress(this.server, this.port)));
        obj.setConnectTimeout(connectTimeout);
        obj.setReadTimeout(this.timeout);
        obj.connect();
        this.doTunneling(obj);
        try {
            return (Socket)HttpConnectSocketImpl.serverSocketField.get(HttpConnectSocketImpl.httpField.get(obj));
        }
        catch (IllegalAccessException cause) {
            throw new InternalError("Should not reach here", cause);
        }
    }
    
    private void doTunneling(HttpURLConnection obj) {
        try {
            HttpConnectSocketImpl.doTunneling.invoke(obj, new Object[0]);
        }
        catch (ReflectiveOperationException cause) {
            throw new InternalError("Should not reach here", cause);
        }
    }
    
    @Override
    protected InetAddress getInetAddress() {
        if (this.external_address != null) {
            return this.external_address.getAddress();
        }
        return super.getInetAddress();
    }
    
    @Override
    protected int getPort() {
        if (this.external_address != null) {
            return this.external_address.getPort();
        }
        return super.getPort();
    }
    
    @Override
    protected int getLocalPort() {
        if (this.socket != null) {
            return super.getLocalPort();
        }
        if (this.external_address != null) {
            return this.external_address.getPort();
        }
        return super.getLocalPort();
    }
    
    static Socket access$200(HttpConnectSocketImpl httpConnectSocketImpl, String s, int n) throws IOException {
        return httpConnectSocketImpl.doTunnel(s, n);
    }
    
    static {
        try {
            Class<Object> forName = Class.forName("sun.net.www.protocol.http.HttpURLConnection", true, null);
            httpField = forName.getDeclaredField("http");
            doTunneling = forName.getDeclaredMethod("doTunneling", new Class[0]);
            serverSocketField = Class.forName("sun.net.NetworkClient", true, null).getDeclaredField("serverSocket");
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    HttpConnectSocketImpl.httpField.setAccessible(true);
                    HttpConnectSocketImpl.serverSocketField.setAccessible(true);
                    return null;
                }
            });
        }
        catch (ReflectiveOperationException cause) {
            throw new InternalError("Should not reach here", cause);
        }
    }
}
